/* 
 * Copyright (C) 1999-2000 Roberto Zunino
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/socket.h> 
#include <netinet/in.h> 

#include "gnome-net-address.h"

/*
 * gnome_net_address_new:
 * @type: the type of network address.
 *
 * Creates a new network address. If you need, you can fill the address
 * octet-by-octet writing it into the %address field of the structure.
 *
 * Returns: a pointer to the new address.
 */
GnomeNetAddress *
gnome_net_address_new(GnomeNetAddressType type)
{
  GnomeNetAddress *a = g_new(GnomeNetAddress, 1);
  
  a->type = type;
  switch (type)
    {
    case GNOME_NET_ADDRESS_INVALID: /* special case */
      a->address_size = 0;
      a->address = NULL;
      return a;
    case GNOME_NET_ADDRESS_INET4:
      a->address_size = sizeof(guint32);
      break;
    case GNOME_NET_ADDRESS_INET6:
      a->address_size = sizeof(guint32)*4;
      break;
    default:
      g_warning("gnome_net_address_new: invalid type=%d", type);
      g_free(a);
      return NULL;
    }

  a->address = g_malloc0(a->address_size);
  return a;
}

/*
 * gnome_net_address_dup:
 * @addr: the address to duplicate.
 *
 * Duplicates a network address, creating a copy.
 *
 * Returns: the duplicate of @addr.
 */
GnomeNetAddress *
gnome_net_address_dup(GnomeNetAddress *addr)
{
  GnomeNetAddress *a = NULL;

  if (! addr)
    return NULL;

  a = g_new(GnomeNetAddress, 1);

  switch (addr->type)
    {
    case GNOME_NET_ADDRESS_INVALID: /* special case */
      a->address_size = 0;
      a->address = NULL;
      return a;
    case GNOME_NET_ADDRESS_INET4:
    case GNOME_NET_ADDRESS_INET6:
      a->type = addr->type;
      a->address_size = addr->address_size;
      a->address = g_memdup(addr->address, addr->address_size);
      return a;
    default:
      g_warning("gnome_net_address_dup: invalid type=%d", addr->type);
      g_free(a);
      return NULL;      
    }
}

/*
 * gnome_net_address_destroy:
 * @addr: the address to destroy.
 *
 * Frees the memory allocated for @addr.
 */
void
gnome_net_address_destroy(GnomeNetAddress *addr)
{
  g_return_if_fail(addr);

  if (addr->address)
    {
      g_free(addr->address);
      addr->address = NULL;
    }

  g_free(addr);
}
