/* EXPERIMENTAL */
#ifndef __GNOME_SOCKET_H__
#define __GNOME_SOCKET_H__

#include <libgnome/gnome-defs.h>
#include <gnome-net-address.h>

typedef enum {
  GNOME_NET_PROTOCOL_TCP = 0,
  GNOME_NET_PROTOCOL_UDP = 1
} GnomeNetProtocol;

typedef enum {
  GNOME_SOCKET_OK             = 0,   /* OK */
  GNOME_SOCKET_JUST_RESOLVED  = 1,   /* DNS lookup succeeded */
  GNOME_SOCKET_JUST_CONNECTED = 2,   /* comm has just been established */
  GNOME_SOCKET_DNS_ERROR      = 3,   /* DNS lookup failed */
  GNOME_SOCKET_SOCKET_ERROR   = 4,   /* socket() failed */
  GNOME_SOCKET_BIND_ERROR     = 5,   /* bind() failed */
  GNOME_SOCKET_LISTEN_ERROR   = 6,   /* listen() failed */
  GNOME_SOCKET_ACCEPT_ERROR   = 7,   /* accept() failed */
  GNOME_SOCKET_CONNECT_ERROR  = 8,   /* connect() failed */
  GNOME_SOCKET_FCNTL_ERROR    = 9,   /* fcntl() failed */
  GNOME_SOCKET_INVALID_PROTO  = 10,  /* invalid parameter */
  /* the following are reserved for higher level abstractions */
  GNOME_SOCKET_READ_ERROR     = 100, /* read() failed */
  GNOME_SOCKET_WRITE_ERROR    = 101, /* write() failed */
  GNOME_SOCKET_CLOSED         = 102  /* remote host has closed the socket */
} GnomeSocketInfo;


/* ***** PLEASE NOTE ******
 * If you don't use ares or GNU adns, you'll have to call gnome_dns_init() 
 * before using these functions (usually in main() after gnome_init()). See
 * http://developer.gnome.org/doc/API/libgnomeui/gnomeui-gnome-dns.html
 * for more info
 * ************************/

typedef void (*GnomeSocketCallback)(guint tag, 
				    GnomeSocketInfo info,
				    gint extra_info,
				    GIOChannel *channel,
				    gpointer data);

guint
gnome_socket_connect (const gchar *hostname, 
		      GnomeNetAddressType addr_type,
		      GnomeNetProtocol proto, guint port,
		      GnomeSocketCallback callback,
		      gpointer callback_data);
guint
gnome_socket_listen(GnomeNetProtocol proto, guint port,
		    GnomeSocketCallback callback,
		    gpointer callback_data,
		    GnomeSocketInfo *info,
		    gint *error_no);

GnomeNetAddress *
gnome_socket_get_addr(guint tag);

guint
gnome_socket_get_port(guint tag);

GnomeSocketCallback
gnome_socket_get_callback(guint tag);

gboolean
gnome_socket_set_callback(guint tag, GnomeSocketCallback callback);

gpointer
gnome_socket_get_callback_data(guint tag);

gboolean
gnome_socket_set_callback_data(guint tag, gpointer callback_data);

gboolean
gnome_socket_close(guint tag);

#endif /* __GNOME_SOCKET_H__ */
